#include <err.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/shm.h>
#include <sys/stat.h>

#include "common_ntpshm.h"

#define NTPSHM_BASE 0x4e545030

struct ntpshm *get_ntpshm(int unit)
{
  int mode;

  if (unit < 2)
    {
      if (getuid() != 0)
	errx(1,"Will not attach to NTPSHM unit %d (< 2) as non-root user.",
	     unit);

      mode = S_IRUSR | S_IWUSR;
    }
  else
    {
      mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
    }

  key_t key = (key_t) (NTPSHM_BASE + unit);

  int shmid = shmget(key, sizeof(struct ntpshm), IPC_CREAT | mode);

  if (shmid == -1)
    err(1,"Failure getting NTPSHM unit %d: shmget(%d,%d,IPC_CREAT | %o)",
	unit, (int) key, (int) sizeof(struct ntpshm), mode);

  struct ntpshm *shm = (struct ntpshm *) shmat(shmid, NULL, 0);

  if (shm == (void *) -1)
    err(1,"Failure attaching NTPSHM unit %d: shmat:",unit);

  return shm;
}
